import re
from .utils.http import Http
from .utils.CONST import HOST


class OpenDataClient:

    def __init__(self, username, password):
        self._username = username
        self._password = password
        Http.login(username=self._username, password=self._password)


    def __getattr__(self, attr):
        if '__' in attr:
            return getattr(self.get, attr)
        return _Callable(self, attr)




class _Executable(object):

    def __init__(self, client, method, path):
        self._client = client
        self._method = method
        self._path = path

    def __call__(self, **kwargs):
        http = Http()
        print(self._method)
        if self._method == 'GET':
            return http.get(self._parse_url, **kwargs)
        if self._method == 'POST': 
            return http.post(self._parse_url, **kwargs)
        

    def __str__(self):
        return '_Executable (%s %s)' % (self._method, self._path)


    @property
    def _parse_url(self):
        url = re.sub('[A-Z]+', lambda x: '/' + x.group().lower(), self._path)
        return HOST + url

    __repr__ = __str__


class _Callable(object):

    def __init__(self, client, name):
        self._client = client
        self._name = name

    def __getattr__(self, attr):
        if attr == 'get':
            return _Executable(self._client, 'GET', self._name)
        if attr == 'post':
            return _Executable(self._client, 'POST', self._name)
        name = '%s/%s' % (self._name, attr)
        return _Callable(self._client, name)

    def __str__(self):
        return '_Callable (%s)' % self._name

    __repr__ = __str__


import os

from setuptools import setup


def _process_requirements():
    packages = open('requirements.txt').read().strip().split('\n')
    requires = []
    for pkg in packages:
        if pkg.startswith('git+ssh'):
            return_code = os.system('pip install {}'.format(pkg))
            assert return_code == 0, 'error, status_code is: {}, exit!'.format(
                return_code)
        else:
            requires.append(pkg)
    return requires


setup(name='jdodata',
      version='1.0.2',
      description="Jianda Open Data",
      long_description="# 简答数据开放数据项目 \n - 用户名密码申请请联系 wukehao@jddatatech.com。",
      classifiers=["Development Status :: 5 - Production/Stable",
                   "License :: OSI Approved :: Apache Software License",
                   "Programming Language :: Python :: 3",
                   "Programming Language :: Python :: 3.5",
                   "Programming Language :: Python :: 3.6",
                   "Programming Language :: Python :: 3.7",
                   "Programming Language :: Python :: 3.8",
                   "Programming Language :: Python :: 3.9",
                   "Programming Language :: Python :: 3.10",
                   "Topic :: Office/Business :: Financial"
                   ],
      keywords='简答数据,jianda,jianda data,open data',
      author='@wukehao',
      author_email='wukehao@jddatatech.com',
      license='MIT',
      packages=['jdodata'],
      include_package_data=True,
      package_data={},
      zip_safe=True,
      install_requires=_process_requirements(),
      entry_points="")
